"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.defaultInflection = exports.newInflector = exports.defaultUtils = undefined;

var _assign = require("babel-runtime/core-js/object/assign");

var _assign2 = _interopRequireDefault(_assign);

var _pluralize = require("pluralize");

var _pluralize2 = _interopRequireDefault(_pluralize);

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var defaultUtils = exports.defaultUtils = {
  constantCase: _utils.constantCase,
  camelCase: _utils.camelCase,
  upperCamelCase: _utils.upperCamelCase,
  pluralize: _pluralize2.default,
  singularize: _pluralize2.default.singular
};

var newInflector = exports.newInflector = function newInflector() {
  var overrides = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : undefined;

  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultUtils,
      constantCase = _ref.constantCase,
      camelCase = _ref.camelCase,
      upperCamelCase = _ref.upperCamelCase,
      pluralize = _ref.pluralize,
      singularize = _ref.singularize;

  return (0, _assign2.default)({
    pluralize,
    argument(name, index) {
      return camelCase(name || `arg${index}`);
    },
    orderByType(typeName) {
      return upperCamelCase(`${pluralize(typeName)}-order-by`);
    },
    orderByEnum(name, ascending, _table, _schema) {
      return constantCase(`${name}_${ascending ? "asc" : "desc"}`);
    },
    domainType(name) {
      return upperCamelCase(name);
    },
    enumName(value) {
      return value;
    },
    enumType(name) {
      return upperCamelCase(name);
    },
    conditionType(typeName) {
      return upperCamelCase(`${typeName}-condition`);
    },
    inputType(typeName) {
      return upperCamelCase(`${typeName}-input`);
    },
    rangeBoundType(typeName) {
      return upperCamelCase(`${typeName}-range-bound`);
    },
    rangeType(typeName) {
      return upperCamelCase(`${typeName}-range`);
    },
    patchType(typeName) {
      return upperCamelCase(`${typeName}-patch`);
    },
    patchField(itemName) {
      return camelCase(`${itemName}-patch`);
    },
    tableName(name, _schema) {
      return camelCase(singularize(name));
    },
    tableNode(name, _schema) {
      return camelCase(singularize(name));
    },
    allRows(name, schema) {
      return camelCase(`all-${this.pluralize(this.tableName(name, schema))}`);
    },
    functionName(name, _schema) {
      return camelCase(name);
    },
    functionPayloadType(name, _schema) {
      return upperCamelCase(`${name}-payload`);
    },
    functionInputType(name, _schema) {
      return upperCamelCase(`${name}-input`);
    },
    tableType(name, schema) {
      return upperCamelCase(this.tableName(name, schema));
    },
    column(name, _table, _schema) {
      return camelCase(name);
    },
    singleRelationByKeys(detailedKeys, table, schema) {
      var _this = this;

      return camelCase(`${this.tableName(table, schema)}-by-${detailedKeys.map(function (key) {
        return _this.column(key.column, key.table, key.schema);
      }).join("-and-")}`);
    },
    updateByKeys(detailedKeys, table, schema) {
      var _this2 = this;

      return camelCase(`update-${this.tableName(table, schema)}-by-${detailedKeys.map(function (key) {
        return _this2.column(key.column, key.table, key.schema);
      }).join("-and-")}`);
    },
    deleteByKeys(detailedKeys, table, schema) {
      var _this3 = this;

      return camelCase(`delete-${this.tableName(table, schema)}-by-${detailedKeys.map(function (key) {
        return _this3.column(key.column, key.table, key.schema);
      }).join("-and-")}`);
    },
    updateNode(name, _schema) {
      return camelCase(`update-${singularize(name)}`);
    },
    deleteNode(name, _schema) {
      return camelCase(`delete-${singularize(name)}`);
    },
    updateByKeysInputType(detailedKeys, name, _schema) {
      var _this4 = this;

      return upperCamelCase(`update-${singularize(name)}-by-${detailedKeys.map(function (key) {
        return _this4.column(key.column, key.table, key.schema);
      }).join("-and-")}-input`);
    },
    deleteByKeysInputType(detailedKeys, name, _schema) {
      var _this5 = this;

      return upperCamelCase(`delete-${singularize(name)}-by-${detailedKeys.map(function (key) {
        return _this5.column(key.column, key.table, key.schema);
      }).join("-and-")}-input`);
    },
    updateNodeInputType(name, _schema) {
      return upperCamelCase(`update-${singularize(name)}-input`);
    },
    deleteNodeInputType(name, _schema) {
      return upperCamelCase(`delete-${singularize(name)}-input`);
    },
    manyRelationByKeys(detailedKeys, table, schema) {
      var _this6 = this;

      return camelCase(`${this.pluralize(this.tableName(table, schema))}-by-${detailedKeys.map(function (key) {
        return _this6.column(key.column, key.table, key.schema);
      }).join("-and-")}`);
    },
    edge(typeName) {
      return upperCamelCase(`${pluralize(typeName)}-edge`);
    },
    edgeField(name, _schema) {
      return camelCase(`${singularize(name)}-edge`);
    },
    connection(typeName) {
      return upperCamelCase(`${this.pluralize(typeName)}-connection`);
    },
    scalarFunctionConnection(procName, _procSchema) {
      return upperCamelCase(`${procName}-connection`);
    },
    scalarFunctionEdge(procName, _procSchema) {
      return upperCamelCase(`${procName}-edge`);
    },
    createField(name, _schema) {
      return camelCase(`create-${singularize(name)}`);
    },
    createInputType(name, _schema) {
      return upperCamelCase(`create-${singularize(name)}-input`);
    },
    createPayloadType(name, _schema) {
      return upperCamelCase(`create-${singularize(name)}-payload`);
    },
    updatePayloadType(name, _schema) {
      return upperCamelCase(`update-${singularize(name)}-payload`);
    },
    deletePayloadType(name, _schema) {
      return upperCamelCase(`delete-${singularize(name)}-payload`);
    }
  }, overrides);
};

var defaultInflection = exports.defaultInflection = newInflector();
//# sourceMappingURL=inflections.js.map