"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function PgConnectionArgCondition(builder, _ref) {
  var inflection = _ref.pgInflection;

  builder.hook("init", function (_, _ref2) {
    var newWithHooks = _ref2.newWithHooks,
        introspectionResultsByKind = _ref2.pgIntrospectionResultsByKind,
        gqlTypeByTypeId = _ref2.pgGqlInputTypeByTypeId,
        _ref2$graphql = _ref2.graphql,
        GraphQLInputObjectType = _ref2$graphql.GraphQLInputObjectType,
        GraphQLString = _ref2$graphql.GraphQLString;

    introspectionResultsByKind.class.map(function (table) {
      var tableTypeName = inflection.tableType(table.name, table.namespace && table.namespace.name);
      /* const TableConditionType = */
      newWithHooks(GraphQLInputObjectType, {
        description: `A condition to be used against \`${tableTypeName}\` object types. All fields are tested for equality and combined with a logical ‘and.’`,
        name: inflection.conditionType(inflection.tableType(table.name, table.namespace && table.namespace.name)),
        fields: function fields(_ref3) {
          var fieldWithHooks = _ref3.fieldWithHooks;
          return introspectionResultsByKind.attribute.filter(function (attr) {
            return attr.classId === table.id;
          }).reduce(function (memo, attr) {
            var fieldName = inflection.column(attr.name, table.name, table.namespace && table.namespace.name);
            memo[fieldName] = fieldWithHooks(fieldName, {
              description: `Checks for equality with the object’s \`${fieldName}\` field.`,
              type: gqlTypeByTypeId[attr.typeId] || GraphQLString
            }, {
              isPgConnectionConditionInputField: true
            });
            return memo;
          }, {});
        }
      }, {
        pgIntrospection: table,
        isPgCondition: true
      });
    });
    return _;
  });
  builder.hook("GraphQLObjectType:fields:field:args", function (args, _ref4, _ref5) {
    var sql = _ref4.pgSql,
        gql2pg = _ref4.gql2pg,
        extend = _ref4.extend,
        getTypeByName = _ref4.getTypeByName,
        introspectionResultsByKind = _ref4.pgIntrospectionResultsByKind;
    var _ref5$scope = _ref5.scope,
        isPgConnectionField = _ref5$scope.isPgConnectionField,
        table = _ref5$scope.pgIntrospection,
        addArgDataGenerator = _ref5.addArgDataGenerator;

    if (!isPgConnectionField || !table || table.kind !== "class") {
      return args;
    }
    var TableConditionType = getTypeByName(inflection.conditionType(inflection.tableType(table.name, table.namespace && table.namespace.name)));

    addArgDataGenerator(function connectionCondition(_ref6) {
      var condition = _ref6.condition;

      return {
        pgQuery: function pgQuery(queryBuilder) {
          if (condition != null) {
            introspectionResultsByKind.attribute.filter(function (attr) {
              return attr.classId === table.id;
            }).forEach(function (attr) {
              var fieldName = inflection.column(attr.name, table.name, table.namespace.name);
              var val = condition[fieldName];
              if (val != null) {
                queryBuilder.where(sql.fragment`${queryBuilder.getTableAlias()}.${sql.identifier(attr.name)} = ${gql2pg(val, attr.type)}`);
              }
            });
          }
        }
      };
    });

    return extend(args, {
      condition: {
        description: "A condition to be used in determining which values should be returned by the collection.",
        type: TableConditionType
      }
    });
  });
};
//# sourceMappingURL=PgConnectionArgCondition.js.map