"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray2 = require("babel-runtime/helpers/slicedToArray");

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _isString = require("lodash/isString");

var _isString2 = _interopRequireDefault(_isString);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function PgMutationPayloadEdgePlugin(builder, _ref) {
  var inflection = _ref.pgInflection;

  builder.hook("GraphQLObjectType:fields", function (fields, _ref2, _ref3) {
    var extend = _ref2.extend,
        getTypeByName = _ref2.getTypeByName,
        sql = _ref2.pgSql;
    var _ref3$scope = _ref3.scope,
        isMutationPayload = _ref3$scope.isMutationPayload,
        pgIntrospection = _ref3$scope.pgIntrospection,
        pgIntrospectionTable = _ref3$scope.pgIntrospectionTable,
        fieldWithHooks = _ref3.fieldWithHooks,
        recurseDataGeneratorsForField = _ref3.recurseDataGeneratorsForField;

    var table = pgIntrospectionTable || pgIntrospection;
    if (!isMutationPayload || !table || table.kind !== "class" || !table.namespace || !table.isSelectable) {
      return fields;
    }
    var tableTypeName = inflection.tableType(table.name, table.namespace.name);
    var TableOrderByType = getTypeByName(inflection.orderByType(tableTypeName));
    var TableEdgeType = getTypeByName(inflection.edge(tableTypeName));
    if (!TableEdgeType) {
      return fields;
    }

    var fieldName = inflection.edgeField(table.name, table.namespace.name);
    recurseDataGeneratorsForField(fieldName);
    return extend(fields, {
      [fieldName]: fieldWithHooks(fieldName, function (_ref4) {
        var addArgDataGenerator = _ref4.addArgDataGenerator;

        addArgDataGenerator(function connectionOrderBy(_ref5) {
          var orderBy = _ref5.orderBy;

          return {
            pgCursorPrefix: orderBy && orderBy.alias && sql.literal(orderBy && orderBy.alias),
            pgQuery: function pgQuery(queryBuilder) {
              if (orderBy != null) {
                var specs = orderBy.specs,
                    unique = orderBy.unique;

                var orders = Array.isArray(specs[0]) ? specs : [specs];
                orders.forEach(function (_ref6) {
                  var _ref7 = (0, _slicedToArray3.default)(_ref6, 2),
                      col = _ref7[0],
                      ascending = _ref7[1];

                  var expr = (0, _isString2.default)(col) ? sql.fragment`${queryBuilder.getTableAlias()}.${sql.identifier(col)}` : col;
                  queryBuilder.orderBy(expr, ascending);
                });
                if (unique) {
                  queryBuilder.setOrderIsUnique();
                }
              }
            }
          };
        });

        var defaultValueEnum = TableOrderByType.getValues().find(function (v) {
          return v.name === "PRIMARY_KEY_ASC";
        }) || TableOrderByType.getValues()[0];
        return {
          description: "An edge for the type. May be used by Relay 1.",
          type: TableEdgeType,
          args: {
            orderBy: {
              description: `The method to use when ordering \`${tableTypeName}\`.`,
              type: TableOrderByType,
              defaultValue: defaultValueEnum && defaultValueEnum.value
            }
          },
          resolve(data) {
            return data.data;
          }
        };
      })
    });
  });
};
//# sourceMappingURL=PgMutationPayloadEdgePlugin.js.map